#include "Ship.h"
#include "StateCore.h"

/*  The Ship class is the parent class and will store all functions that are not
	specific to any one ship and it will store virtual functions that will be
	defined in child classes.  Any data that is ship specific will be listed in
	the child class while common data such as speed will be stored in the parent
	class as protected with a Get function for retrieving the data.*/

/*  Ships can be created by listing the X and Y distance from the center of the ship
	in the constructor of the child class.
	If ships take up 32x32 then center would be x=15, y=15
	List the vertices either in a clockwise or anticlockwise manner, as long as each
	point connects, the DrawShip function in Game will connect the last point in the
	list to the first point. */

Ship::Ship(StateCore &core)
	:
	Entity(core)
{
}

Ship::~Ship()
{
}

//  ------------End of Ship class-------------


Player::Player(Input &user, StateCore &core)
	:
	Ship(core),
	user(user),
	shield(core),
	wc(core)
{
	velocity = 350;// Pixels per second
	numVerts = 4;
	radius = 16;
	x = 400;
	y = 300;
	
	health = 1.0f;
	damage = 0.20f;
	lives = 3;
	fireTime = 0.0f;
	energy = 0;
	verts = new Vertices[numVerts];

	// Point 1
	verts[0].vx = 0;
	verts[0].vy = -15;

	// Point 2
	verts[1].vx = -10;
	verts[1].vy = 15;

	// Point 3
	verts[2].vx = 0;
	verts[2].vy = 10;

	// Point 4
	verts[3].vx = 10;
	verts[3].vy = 15;
}

void Player::Fire(Coord pos)
{
	wc.Fire(pos);	
}

void Player::Draw()
{
	for(int i = 0; i < numVerts; i++)
	{
		int j = (i+1) % numVerts;
		core.gfx.DrawShip(x, y, verts[i].vx, verts[i].vy, verts[j].vx, verts[j].vy, core.gfx.FILLCOLOR_XRGB(255, 0, 255));		
	}
}

void Player::Move( )
{
	int scrnX = 0;
	int scrnY = 0;
	bool pause;
	user.GetAxes((float*)&scrnX, (float *)&scrnY);
	scrnX > 0 ? x += min(5, CalcSpeed(velocity) * moveTimer) : x += max(-5, CalcSpeed(velocity) * moveTimer);
	scrnY > 0 ? y += min(5, CalcSpeed(velocity) * moveTimer) : y += max(-5, CalcSpeed(velocity) * moveTimer);
}

void Player::DoCollision( Entity *ent )
{
	if(shield.GetHealth() > 0.0f)
	{
		shield.ReduceHealth(ent->GetDamage() );
	}
	else
	{
		ReduceHealth(ent->GetDamage() );
	}
}

void Player::Die()
{
	if(lives > 0)
	{
		shield.ReduceHealth( -1.0f);
		health = 1.0f;
		lives--;
		SetPosition(SCREENWIDTH / 2, SCREENHEIGHT / 2);
	}
}

void Player::DecideState()
{
	if(health <= 0.0f)
	{
		state = Dead;
	}
	else
	{
		state = Moving;
		if(user.Firing() )
		{
			state = Firing;
		}
	}
}

WeaponController *Player::GetWC()
{
	return &wc;
}

void Player::Do()
{
	DecideState(  );
	Coord pos = {x, y};
	switch(state)
	{
	case Firing:
		Fire(pos);								// No break; on purpose.  When player fires ship should also move same frame
	case Moving:
		Move();
		shield.Do();
		break;
	case Dead:
		Die();
		break;
	}
}

void Player::Clip()
{
	unsigned int left = 0;
	unsigned int top = 0;
	unsigned int right = core.gfx.GetWidth();
	unsigned int bottom = core.gfx.GetHeight();

	if(x <=  left + radius )
	{
		x = left + radius;
	}
	if(y <= top + radius )
	{
		y = top + radius;
	}
	if(x > right - radius)
	{
		x = right - radius;
	}
	if(y > bottom - radius)
	{
		y = bottom - radius;
	}
}

Player::~Player()
{

}

//  End of Player Ship Class

Enemy::Enemy(StateCore &core)
	:
	Ship(core),
	Nextp( NULL ),
	CurPoint( 1 )
{
	velocity = 180;
	maxSpeed = 3;
	radius = 32;
	health = 0.50f;
	scoreValue = 10;
	damage = 0.20f;
	hitByAmmo = false;
	state = Moving;

}

void Enemy::CalculatePath(Ship::Vertices* ePos, const Ship::Vertices* pPos)
{
	rise = CalcRise( ePos->vy, pPos->vy );
	run = CalcRun( ePos->vx, pPos->vx );
	slope = CalcSlope( rise, run );
	float nrmlSpeed = CalcSpeed( velocity);
	nrmlSpeed = min(nrmlSpeed, maxSpeed);
	if(rise > run)
	{
		if(ePos->vx < pPos->vx)
		{
			ePos->vx += slope * nrmlSpeed;
		}
		else if(ePos->vx > pPos->vx)
		{
			ePos->vx -= slope * nrmlSpeed;
		}
		if(ePos->vy < pPos->vy)
		{
			ePos->vy += nrmlSpeed;
		}
		else if(ePos->vy > pPos->vy)
		{
			ePos->vy -= nrmlSpeed;
		}
	}
	else if(rise < run)
	{
		if(ePos->vx < pPos->vx)
		{
			ePos->vx += nrmlSpeed;
		}
		else if(ePos->vx > pPos->vx)
		{
			ePos->vx -= nrmlSpeed;
		}
		if(ePos->vy < pPos->vy)
		{
			ePos->vy += slope * nrmlSpeed;
		}
		else if(ePos->vy > pPos->vy)
		{
			ePos->vy -= slope * nrmlSpeed;
		}
	}
	else
	{
		if( ePos->vx < pPos->vx )
		{
			ePos->vx += nrmlSpeed;
		}
		else if( ePos->vx > pPos->vx)
		{
			ePos->vx -= nrmlSpeed;
		}
		if( ePos->vy < pPos->vy )
		{
			ePos->vy += nrmlSpeed;
		}
		else if( ePos->vy > pPos->vy )
		{
			ePos->vy -= nrmlSpeed;
		}
	}
}

void Enemy::CheckforWayPoint( )
{	
	dist = sqrt( ( rise * rise ) + ( run* run ) );
	if( dist < 15 )
	{
		if( CurPoint < NPoints - 1 )
		{
			CurPoint++;
			Nextp = &Points[ CurPoint ];
		}
		else
		{
			health = 0;
		}
	}

}

void Enemy::Move()
{
	Vertices ePos = {x, y};

	CalculatePath(&ePos, Nextp);
	SetPosition(ePos.vx, ePos.vy);
}

void Enemy::Fire()
{
	Coord pos = {x, y};
}

void Enemy::Die()
{
}

void Enemy::Do()
{
	switch(state)
	{
		case Moving:
			Move();
			CheckforWayPoint();
		break;
	}
}

void Enemy::SetSpawnPoint( unsigned int X, unsigned int Y )
{	
	for(unsigned int i = 0; i < NPoints; i++)
	{
		Points[ i ].vy += Y;
	}
	SetPosition( X, Y );
}

void Enemy::AdjustWaypoints( float offSet )
{
	for(unsigned int i = 1; i < NPoints; i++)
	{
		Points[ i ].vy -= offSet;
	}
}

Enemy::~Enemy()
{
}
// End of Enemy Base Class

Enemy1::Enemy1(StateCore &core)
	:
	Enemy(core)
{
	maxSpeed = 3;
	NPoints = 3;
	Points = new Vertices[ NPoints ];
	Points[ 0 ].vx = 0;
	Points[ 0 ].vy = 0;
	Points[ 1 ].vx = SCREENWIDTH;
	Points[ 1 ].vy = 0;
	Points[ 2 ].vx = SCREENWIDTH / 2;
	Points[ 2 ].vy = SCREENHEIGHT + radius;
	Nextp = &Points[ 1 ];

	numVerts = 4;
	verts = new Vertices[numVerts];
	// Point 1
	verts[0].vx = 0;
	verts[0].vy = -15;
	// Point 2
	verts[1].vx = -10;
	verts[1].vy = 0;
	// Point 3
	verts[2].vx = 0;
	verts[2].vy = 15;
	// Point 4
	verts[3].vx = 10;
	verts[3].vy = 0;
}

Enemy2::Enemy2(StateCore &core)
	:
	Enemy(core)
{
	maxSpeed = 3;
	NPoints = 3;
	Points = new Vertices[ NPoints ];
	x = 0;
	y = 0;
	Points[ 1 ].vx = SCREENWIDTH / 2;
	Points[ 1 ].vy = SCREENHEIGHT;
	Points[ 2 ].vx = SCREENWIDTH + (radius * 2);
	Points[ 2 ].vy = 0 - (radius * 2);
	Nextp = &Points[ 1 ];

	numVerts = 4;
	verts = new Vertices[numVerts];
	// Point 1
	verts[0].vx = -15;
	verts[0].vy = -15;
	// Point 2
	verts[1].vx = 15;
	verts[1].vy = -15;
	// Point 3
	verts[2].vx = 15;
	verts[2].vy = 15;
	// Point 4
	verts[3].vx = -15;
	verts[3].vy = 15;
}

Enemy3::Enemy3(StateCore &core)
	:
	Enemy(core)
{
	maxSpeed = 3;
	NPoints = 2;
	Points = new Vertices[ NPoints ];
	x = 0;
	y = 0;
	Points[ 1 ].vx = SCREENWIDTH + (radius * 2 );
	Points[ 1 ].vy = SCREENHEIGHT + (radius * 2 );
	Nextp = &Points[ 1 ];

	numVerts = 8;
	verts = new Vertices[numVerts];
	// Point 1
	verts[0].vx = -7;
	verts[0].vy = -15;
	// Point 2
	verts[1].vx = 7;
	verts[1].vy = -15;
	// Point 3
	verts[2].vx = 16;
	verts[2].vy = -7;
	// Point 4
	verts[3].vx = 16;
	verts[3].vy = 7;
	// Point 5
	verts[4].vx = 7;
	verts[4].vy = 16;
	//Point 6
	verts[5].vx = -7;
	verts[5].vy = 16;
	//Point 7
	verts[6].vx = -15;
	verts[6].vy = 7;
	//Point 8
	verts[7].vx = -15;
	verts[7].vy = -7;
}

Enemy4::Enemy4(StateCore &core)
	:
	Enemy(core)
{
	maxSpeed = 3;
	NPoints = 7;
	Points = new Vertices[ NPoints ];
	Points[ 1 ].vx = ( SCREENWIDTH / 2 ) + 50;
	Points[ 1 ].vy = 100;
	Points[ 2 ].vx = ( SCREENWIDTH / 2 ) - 50;
	Points[ 2 ].vy = Points[ 1 ].vy + 100;
	Points[ 3 ].vx = Points[ 1 ].vx;
	Points[ 3 ].vy = Points[ 2 ].vy + 100;
	Points[ 4 ].vx = Points[ 2 ].vx;
	Points[ 4 ].vy = Points[ 3 ].vy + 100;
	Points[ 5 ].vx = Points[ 1 ].vx;
	Points[ 5 ].vy = Points[ 3 ].vy + 100;
	Points[ 6 ].vx = Points[ 2 ].vx;
	Points[ 6 ].vy = Points[ 4 ].vy + 100 + (radius * 2);
	Nextp = &Points[ 1 ];

	numVerts = 7;
	verts = new Vertices[numVerts];
	// Point 1
	verts[0].vx = -5;
	verts[0].vy = -15;
	// Point 2
	verts[1].vx = 0;
	verts[1].vy = -5;
	// Point 3
	verts[2].vx = -11;
	verts[2].vy = -5;
	// Point 4
	verts[3].vx = 0;
	verts[3].vy = 16;
	// Point 5
	verts[4].vx = 11;
	verts[4].vy = -5;
	// Point 6
	verts[5] = verts[1];
	// Point 7
	verts[6].vx = 5;
	verts[6].vy = -15;
}

Enemy5::Enemy5(StateCore &core)
	:
	Enemy(core)
{
	maxSpeed = 3;
	NPoints = 5;
	Points = new Vertices[ NPoints ];
	Points[ 1 ].vx = 0;
	Points[ 1 ].vy = SCREENHEIGHT / 2;
	Points[ 2 ].vx = SCREENWIDTH / 2;
	Points[ 2 ].vy = SCREENHEIGHT;
	Points[ 3 ].vx = SCREENWIDTH;
	Points[ 3 ].vy = SCREENHEIGHT / 2;
	Points[ 4 ].vx = SCREENWIDTH / 2 + (radius * 2);
	Points[ 4 ].vy = 0 - (radius * 2);
	Nextp = &Points[ 1 ];

	numVerts = 10;
	verts = new Vertices[numVerts];
	// Point 1
	verts[0].vx = -9;
	verts[0].vy = -15;
	// Point 2
	verts[1].vx = -6;
	verts[1].vy = -3;
	// Point 3
	verts[2].vx = -15;
	verts[2].vy = 4;
	// Point 4
	verts[3].vx = -3;
	verts[3].vy = 4;
	// Point 5
	verts[4].vx = 0;
	verts[4].vy = 16;
	// Point 6
	verts[5].vx = 4;
	verts[5].vy = 4;
	// Point 7
	verts[6].vx = 16;
	verts[6].vy = 4;
	// Point 8
	verts[7].vx = 7;
	verts[7].vy = -3;
	// Point 9
	verts[8].vx = 10;
	verts[8].vy = -15;
	// Point 10
	verts[9].vx = 0;
	verts[9].vy = -8;
}
// End of Enemy Derived classes

Boss1::Boss1(StateCore &core)
	:
	Enemy(core),
	BState( Attack ),
	FireRate(0.5f),
	RegenRate(3.0f),
	LaunchTimer(0),
	RegenTimer(0),
	mCount(0),
	NMissiles(10),
	shield(core)
{
	health = 10;
	shield.ReduceHealth( -1.0f );
	radius = 16;
	shield.SetRadius( 40);
	scoreValue = 1000;
	numVerts = 4;
	verts = new Vertices[numVerts];
	// Point 1
	verts[0].vx = 0;
	verts[0].vy = -15;
	// Point 2
	verts[1].vx = -10;
	verts[1].vy = 0;
	// Point 3
	verts[2].vx = 0;
	verts[2].vy = 15;
	// Point 4
	verts[3].vx = 10;
	verts[3].vy = 0;

}

void Boss1::DecideState()
{
	readyToFire = false;
	if(mCount < NMissiles )
	{
		BState = Attack;

		if(LaunchTimer < FireRate * 1000)
		{
			BState = Idle;
		}
	}
	else
	{
		if(RegenTimer < RegenRate * 1000)
		{
			BState = RegenMissiles;
		}
		else
		{
			mCount %= NMissiles;
		}
	}
}

void Boss1::Do( )
{
	DecideState();

	switch( BState )
	{
	case Attack:
		readyToFire = true;
		LaunchTimer = 0;
		Fire();
		mCount++;
		RegenTimer = 0;
		break;
	case RegenMissiles:
		UpdateTime( &RegenTimer );
		break;
	case Idle:
		UpdateTime( &LaunchTimer );
		break;
	}
}

//bool Boss1::MissileExploded(unsigned int i)
//{
//	
//	return Missiles[ i ].HasExploded();
//}

//int Boss1::GetMissileRad()
//{
//	return Missiles[0].GetMIR();
//}

void Boss1::Fire()
{
	// needs player position
	Coord pos = {0};	
}

// Boss2 class
Boss2::Boss2(StateCore &core )
	: 
	Enemy(core),
	px( 0 ),
	py( 0 ),
	ramCooldown( 3 ),
	bossState( None )
{
	velocity = 800;
	maxSpeed = 13;
	NPoints = 1;
	Points = new Vertices[ NPoints ];
	Points[ 0 ].vx = 0;
	Points[ 0 ].vy = 0;
	Nextp = &Points[ 0 ];

	numVerts = 4;
	verts = new Vertices[numVerts];
	// Point 1
	verts[0].vx = 0;
	verts[0].vy = -15;
	// Point 2
	verts[1].vx = -10;
	verts[1].vy = 0;
	// Point 3
	verts[2].vx = 0;
	verts[2].vy = 15;
	// Point 4
	verts[3].vx = 10;
	

}

void Boss2::Do()
{
	switch(bossState)
	{
		case Raming:
			Move();
			CheckforWayPoint();
			break;
		case None:
			TransitionState();
			break;
	}
}

void Boss2::TransitionState()
{
	if( ramCooldownTime >= ramCooldown * 1000 )
	{
		ramCooldownTime = 0;
		bossState = Raming;
		Points[ 0 ].vx = px;
		Points[ 0 ].vy = py;
	}
}

void Boss2::CheckforWayPoint( )
{
	dist = sqrt( ( rise * rise ) + ( run * run ) );
	if( dist < 15 )
	{
		bossState = None;
		ramCooldownTime = 0;
	}
}

float Boss2::GetPlayerX()
{
	return px;
}

float Boss2::GetPlayerY()
{
	return py;
}

void Boss2::Draw()
{
	for(int i = 0; i < numVerts; i++)
	{
		int j = (i+1) % numVerts;
		core.gfx.DrawShip( x, y, verts[i].vx, verts[i].vy, verts[j].vx, verts[j].vy, core.gfx.FILLCOLOR_XRGB( 255,255,255 ) );
	}
	core.gfx.DrawLine( x, y, px, py, core.gfx.FILLCOLOR_XRGB( 0,255,0 ) );

}